function varargout = hh_gui(varargin)
% HH_GUI M-file for hh_gui.fig
%      HH_GUI, by itself, creates a new HH_GUI or raises the existing
%      singleton*.
%
%      H = HH_GUI returns the handle to a new HH_GUI or the handle to
%      the existing singleton*.
%
%      HH_GUI('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in HH_GUI.M with the given input arguments.
%
%      HH_GUI('Property','Value',...) creates a new HH_GUI or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before hh_gui_OpeningFunction gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to hh_gui_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Copyright 2002-2003 The MathWorks, Inc.

% Edit the above text to modify the response to help hh_gui

% Last Modified by GUIDE v2.5 15-Jan-2007 15:43:14

% Interface to solve the HH equations and plot:
%   Activation variables    figure(1)  v, n, m and h
%   Currents                figure(2)  v, I_Na, I_K and I_l
%
% PD Roberts, BME-OHSU 01/07 (modified from: Steve Cox, 1/17/03)
% Port to MATLAB GUI by Gerardo Lafferriere, Math-PSU, 1/15/07.
%

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @hh_gui_OpeningFcn, ...
                   'gui_OutputFcn',  @hh_gui_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before hh_gui is made visible.
function hh_gui_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to hh_gui (see VARARGIN)

% Choose default command line output for hh_gui
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);
% UIWAIT makes hh_gui wait for user response (see UIRESUME)
% uiwait(handles.figure1);

%--- Set model parameters -------------------------------------
tstop = 20;  % Duration of simulation (msec)

% parameters for Hodgkin-Huxley equations (from table 3 of [Hodgkin53])
GNa = 120;           % Maximum sodium current density (mS / cm^2)
GK = 36;             % Maximum potassium current density (mS / cm^2)
Gl = 0.3;            % Leak current density (mS / cm^2)
vNa = 50;            % Reversal potential for sodium current (mV)
vK = -77;            % Reversal potential for potassium current (mV)
vl = -54.3;          % Reversal potential for leak current (mV)

% stimulus parameters
del = 5;             % Delay until beginning of stimulus (msec)
dur = 1;            % Duration of stimulus (msec)
amp = 0.1;           % Amplitude of stimulus (nA)
% Initialize fields in the GUI interface.  See also the RESET function.
set(handles.simu_length,'String',num2str(tstop));
set(handles.sodium_current,'String',num2str(GNa));
set(handles.potassium_current,'String',num2str(GK));
set(handles.leak_current,'String',num2str(Gl));
set(handles.sodium_reversal,'String',num2str(vNa));
set(handles.potassium_reversal,'String',num2str(vK));
set(handles.leak_reversal,'String',num2str(vl));
set(handles.stim_del,'String',num2str(del));
set(handles.stim_dur,'String',num2str(dur));
set(handles.stim_amp,'String',num2str(amp));
%guidata(hObject, handles);



% --- Outputs from this function are returned to the command line.
function varargout = hh_gui_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;



function sodium_current_Callback(hObject, eventdata, handles)
% hObject    handle to sodium_current (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of sodium_current as text
%        str2double(get(hObject,'String')) returns contents of sodium_current as a double


% --- Executes during object creation, after setting all properties.
function sodium_current_CreateFcn(hObject, eventdata, handles)
% hObject    handle to sodium_current (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function leak_current_Callback(hObject, eventdata, handles)
% hObject    handle to leak_current (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of leak_current as text
%        str2double(get(hObject,'String')) returns contents of leak_current as a double


% --- Executes during object creation, after setting all properties.
function leak_current_CreateFcn(hObject, eventdata, handles)
% hObject    handle to leak_current (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function potassium_current_Callback(hObject, eventdata, handles)
% hObject    handle to potassium_current (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of potassium_current as text
%        str2double(get(hObject,'String')) returns contents of potassium_current as a double


% --- Executes during object creation, after setting all properties.
function potassium_current_CreateFcn(hObject, eventdata, handles)
% hObject    handle to potassium_current (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function leak_reversal_Callback(hObject, eventdata, handles)
% hObject    handle to leak_reversal (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of leak_reversal as text
%        str2double(get(hObject,'String')) returns contents of leak_reversal as a double


% --- Executes during object creation, after setting all properties.
function leak_reversal_CreateFcn(hObject, eventdata, handles)
% hObject    handle to leak_reversal (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function potassium_reversal_Callback(hObject, eventdata, handles)
% hObject    handle to potassium_reversal (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of potassium_reversal as text
%        str2double(get(hObject,'String')) returns contents of potassium_reversal as a double


% --- Executes during object creation, after setting all properties.
function potassium_reversal_CreateFcn(hObject, eventdata, handles)
% hObject    handle to potassium_reversal (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function sodium_reversal_Callback(hObject, eventdata, handles)
% hObject    handle to sodium_reversal (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of sodium_reversal as text
%        str2double(get(hObject,'String')) returns contents of sodium_reversal as a double


% --- Executes during object creation, after setting all properties.
function sodium_reversal_CreateFcn(hObject, eventdata, handles)
% hObject    handle to sodium_reversal (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function simu_length_Callback(hObject, eventdata, handles)
% hObject    handle to simu_length (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of simu_length as text
%        str2double(get(hObject,'String')) returns contents of simu_length as a double


% --- Executes during object creation, after setting all properties.
function simu_length_CreateFcn(hObject, eventdata, handles)
% hObject    handle to simu_length (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function stim_amp_Callback(hObject, eventdata, handles)
% hObject    handle to stim_amp (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of stim_amp as text
%        str2double(get(hObject,'String')) returns contents of stim_amp as a double


% --- Executes during object creation, after setting all properties.
function stim_amp_CreateFcn(hObject, eventdata, handles)
% hObject    handle to stim_amp (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function stim_dur_Callback(hObject, eventdata, handles)
% hObject    handle to stim_dur (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of stim_dur as text
%        str2double(get(hObject,'String')) returns contents of stim_dur as a double


% --- Executes during object creation, after setting all properties.
function stim_dur_CreateFcn(hObject, eventdata, handles)
% hObject    handle to stim_dur (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function stim_del_Callback(hObject, eventdata, handles)
% hObject    handle to stim_del (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of stim_del as text
%        str2double(get(hObject,'String')) returns contents of stim_del as a double


% --- Executes during object creation, after setting all properties.
function stim_del_CreateFcn(hObject, eventdata, handles)
% hObject    handle to stim_del (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in run_button.
function run_button_Callback(hObject, eventdata, handles)
% hObject    handle to run_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Read the current values of the parameters from the edit fields and put
% them in easier to read variables.
GNa = str2num(get(handles.sodium_current,'String'));
GK = str2num(get(handles.potassium_current,'String'));
Gl = str2num(get(handles.leak_current,'String'));
vNa = str2num(get(handles.sodium_reversal,'String'));
vK = str2num(get(handles.potassium_reversal,'String'));
vl = str2num(get(handles.leak_reversal,'String'));
HH_param = [GNa GK Gl vNa vK vl];  % Load parameter vector
del = str2num(get(handles.stim_del,'String'));
dur = str2num(get(handles.stim_dur,'String'));
amp = str2num(get(handles.stim_amp,'String'));
stim_param = [del dur amp];  % Load parameter vector
% load simulation duration
tstop =  str2num(get(handles.simu_length,'String'));

%--- Compute variables -------------------------------------------
[t,x] = hh_integrate(tstop, HH_param, stim_param);  % Call integrator function  

INa = GNa*(x(:,3).^3).*x(:,4).*(x(:,1)-vNa); % Sodium current for plot
IK = GK*(x(:,2).^4).*(x(:,1)-vK);   % Potassium current for plot
Il = Gl*(x(:,1)-vl);                % Leak current for plot

%--- Plotting functions -------------------------------------------
% Plot membrane potential and activation variables
scrsz = get(0,'ScreenSize'); % Find out your screen size to adjust the size
                             % of the plotting window 
figure(1)
set(1,'Position',[10 scrsz(4)/4 scrsz(3)/1.8 scrsz(4)/1.8])
set(1,'Name','Membrane potential and Activation Variables')
subplot(2,2,1)
plot(t,x(:,1), 'k')
ylabel('v (mV)','fontsize',16)
subplot(2,2,2)
plot(t,x(:,2), 'r')
ylabel('n','fontsize',16)
subplot(2,2,3)
plot(t,x(:,3), 'b')
xlabel('t (ms)','fontsize',16)
ylabel('m','fontsize',16)
subplot(2,2,4)
plot(t,x(:,4), 'g')
xlabel('t (ms)','fontsize',16)
ylabel('h','fontsize',16)

% Plot membrane potential and currents
figure(2)
set(2,'Position',[20 50 scrsz(3)/1.8 scrsz(4)/1.8])
set(2,'Name','Membrane potential and Currents')
subplot(2,2,1)
plot(t,x(:,1), 'k')
ylabel('v (mV)','fontsize',16)
subplot(2,2,2)
plot(t,INa, 'r')
ylabel('I_{Na}','fontsize',16)
subplot(2,2,3)
plot(t,IK, 'b')
xlabel('t (ms)','fontsize',16)
ylabel('I_K','fontsize',16)
subplot(2,2,4)
plot(t,Il, 'y')
xlabel('t (ms)','fontsize',16)
ylabel('I_l','fontsize',16)

% End main run routine.

%%%%% ---------- hh_integrate ------------ %%%%%
%
% hh_integrate.m
%
% Integrate the Hodgkin Huxley equations
%     Function takes parameters from the interface and uses an ODE
%     integrator to solve the nested equations. 
%
% PD Roberts, BME-OHSU 01/07 (modified from: Steve Cox, 1/17/03)
%
function [t,x] = hh_integrate(t, hh_param, stim_param)

X0 = [-65 0.318 0.053 0.595]';  % initial valuse of [v n m h]
[t,x] = ode23(@hhp,[0 t],X0,[],hh_param,stim_param);  % Runge-Kutta algorithm


% Set up the Hodgkin-Huxley equations 
% Dynamic variables are stored in X = [v n m h]'

%--- Hodgkin-Huxley equations voltage equation, pp 173 of [Dayan01] ---
function dXdt = hhp(t,X,hh_param,stim_param)

    dXdt = zeros(4,1);

    GNa = hh_param(1);  
    GK = hh_param(2);    
    Gl = hh_param(3);   
    vNa = hh_param(4); 
    vK = hh_param(5);
    vl = hh_param(6);
    Cm = 1;              % capacitance density (uF / cm^2)
    A = 1.26e-5;	 % membrane area (A=4*PI*R^2, R=10u, cm^2)
 
    %  stimulus is current injection
    I = 0;
    if (t>=stim_param(1) & t<=stim_param(1)+stim_param(2))
       I = stim_param(3)/1000;  % parameter in nA
    end

    % right hand sides of HH equations (26),(7),(15),(16) of [Hodgkin52]

    dXdt(1) = (1/Cm)*(-GK*(X(2)^4)*(X(1)-vK) + ...
           -GNa*(X(3)^3)*X(4)*(X(1)-vNa) - Gl*(X(1)-vl) + I/A);

    dXdt(2) = an(X(1))*(1-X(2)) - bn(X(1))*X(2);

    dXdt(3) = am(X(1))*(1-X(3)) - bm(X(1))*X(3);

    dXdt(4) = ah(X(1))*(1-X(4)) - bh(X(1))*X(4);

    return
% hhp

%--- Voltage dependent rate functions from  pp 171-172 of [Dayan01] ---

function val = an(v)  % opening rate functions of potassium activation
    val = 0.01*(v+55)/(1-exp(-0.1*(v+55)));
% end al.

function val = bn(v)  % closing rate functions of potassium activation
    val = 0.125*exp(-0.0125*(v+65));
% end bn

function val = am(v)  % opening rate functions of sodium activation
    val = .1*(v+40)/(1-exp(-.1*(v+40)));
% end am

function val = bm(v)  % closing rate functions of sodium activation
    val = 4*exp(-0.0556*(v+65));
% end bm

function val = ah(v)  % opening rate functions of sodium inactivation
    val = 0.07*exp(-0.05*(v+65));
% end ah

function val = bh(v)  % closing rate functions of sodium inactivation
    val = 1/(1+exp(-0.1*(v+35)));
% end bh






% --- Executes on button press in reset_button.
function reset_button_Callback(hObject, eventdata, handles)
% hObject    handle to reset_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% If this button is pressed we reset to the initialization values.
tstop = 20;  % Duration of simulation (msec)

% parameters for Hodgkin-Huxley equations (from table 3 of [Hodgkin53])
GNa = 120;           % Maximum sodium current density (mS / cm^2)
GK = 36;             % Maximum potassium current density (mS / cm^2)
Gl = 0.3;            % Leak current density (mS / cm^2)
vNa = 50;            % Reversal potential for sodium current (mV)
vK = -77;            % Reversal potential for potassium current (mV)
vl = -54.3;          % Reversal potential for leak current (mV)

% stimulus parameters
del = 5;             % Delay until beginning of stimulus (msec)
dur = 1;            % Duration of stimulus (msec)
amp = 0.1;           % Amplitude of stimulus (nA)

set(handles.simu_length,'String',num2str(tstop));
set(handles.sodium_current,'String',num2str(GNa));
set(handles.potassium_current,'String',num2str(GK));
set(handles.leak_current,'String',num2str(Gl));
set(handles.sodium_reversal,'String',num2str(vNa));
set(handles.potassium_reversal,'String',num2str(vK));
set(handles.leak_reversal,'String',num2str(vl));
set(handles.stim_del,'String',num2str(del));
set(handles.stim_dur,'String',num2str(dur));
set(handles.stim_amp,'String',num2str(amp));


